@@ -1,5 +1,8 @@
 """Top-level entry points to yakonfig.
 
+.. This software is released under an MIT/X11 open source license.
+   Copyright 2014 Diffeo, Inc.
+
 Purpose
 =======
 
@@ -9,22 +12,14 @@ Most programs' `main()` functions will call yakonfig as:
 >>> yakonfig.parse_args(parser, [yakonfig, module, module...])
 
 where the list of modules are top-level modules or other
-`yakonfig.Configurable` objects the program uses.
+:class:`yakonfig.Configurable` objects the program uses.
 
 Test code and other things not driven by argparse can instead call
 
 >>> yakonfig.set_default_config([yakonfig, module, module, ...])
 
-
-Implementation Details
-======================
-
-
------
-
-This software is released under an MIT/X11 open source license.
-
-Copyright 2014 Diffeo, Inc.
+Module Contents
+===============
 
 """
 
@@ -32,41 +27,58 @@ from __future__ import absolute_import
 import collections
 import contextlib
 from cStringIO import StringIO
+import sys
 
 import yaml
 
 import yakonfig
-from yakonfig.merge import overlay_config
+from yakonfig.merge import overlay_config, diff_config
 from yakonfig.yakonfig import _temporary_config
 
 # These implement the Configurable interface for yakonfig proper!
 config_name = 'yakonfig'
 def add_arguments(parser):
+    '''Add command-line arguments for yakonfig proper.
+
+    This is part of the :class:`~yakonfig.Configurable` interface, and
+    is usually run by including :mod:`yakonfig` in the
+    :func:`parse_args()` module list.
+
+    :param argparse.ArgumentParser parser: command-line argument
+      parser
+
+    '''
     parser.add_argument('--config', '-c', metavar='FILE',
                         help='read configuration from FILE')
+    parser.add_argument('--dump-config', metavar='WHAT', nargs='?',
+                        help='dump out configuration then stop '
+                        '(default, effective, full)')
 runtime_keys = { 'config': 'config' }
 
 def parse_args(parser, modules, args=None):
     """Set up global configuration for command-line tools.
 
-    `modules` is an iterable of `yakonfig.Configurable` objects, or
-    anything equivalently typed.  This function iterates through those
-    objects and calls `add_arguments()` on each to build up a complete
-    list of command-line arguments, then calls
-    `argparse.ArgumentParser.parse_args()` to actually process the
-    command line.  This produces a configuration that is a combination
-    of all default values declared by all modules; configuration
-    specified in `--config` arguments; and overriding configuration
-    values specified in command-line arguments.
+    `modules` is an iterable of
+    :class:`yakonfig.Configurable` objects, or anything
+    equivalently typed.  This function iterates through those objects
+    and calls
+    :meth:`~yakonfig.Configurable.add_arguments` on
+    each to build up a complete list of command-line arguments, then
+    calls :meth:`argparse.ArgumentParser.parse_args` to actually
+    process the command line.  This produces a configuration that is a
+    combination of all default values declared by all modules;
+    configuration specified in ``--config`` arguments; and overriding
+    configuration values specified in command-line arguments.
 
-    This returns the ArgumentParser Namespace object, in case the
+    This returns the :class:`argparse.Namespace` object, in case the
     application has defined its own command-line parameters and
     needs to process them.  The new global configuration can be
-    obtained via `yakonfig.get_global_config()`.
+    obtained via :func:`yakonfig.get_global_config`.
 
     :param argparse.ArgumentParser parser: application-provided
       argument parser
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.Configurable`
     :param args: command-line options, or `None` to use `sys.argv`
     :return: the new global configuration
 
@@ -75,6 +87,16 @@ def parse_args(parser, modules, args=None):
     namespace = parser.parse_args(args)
     try:
         yakonfig.set_default_config(modules, params=vars(namespace))
+        if getattr(namespace, 'dump_config', None):
+            if namespace.dump_config == 'full':
+                to_dump = yakonfig.get_global_config()
+            elif namespace.dump_config == 'default':
+                to_dump = assemble_default_config(modules)
+            else: # 'effective'
+                to_dump = diff_config(assemble_default_config(modules),
+                                      yakonfig.get_global_config())
+            yaml.dump(to_dump, sys.stdout)
+            parser.exit()
     except yakonfig.ConfigurationError, e:
         parser.error(e)
     return namespace
@@ -83,20 +105,23 @@ def set_default_config(modules, params={}, yaml=None, filename=None,
                        config=None, validate=True):
     """Set up global configuration for tests and noninteractive tools.
 
-    `modules` is an iterable of `yakonfig.Configurable' objects, or
-    anything equivalently typed.  This function iterates through those
-    objects to produce a default configuration, reads `yaml` as though
-    it were the configuration file, and fills in any values from
-    `params` as though they were command-line arguments.  The
-    resulting configuration is set as the global configuration.
+    `modules` is an iterable of
+    :class:`yakonfig.Configurable` objects, or anything
+    equivalently typed.  This function iterates through those objects
+    to produce a default configuration, reads `yaml` as though it were
+    the configuration file, and fills in any values from `params` as
+    though they were command-line arguments.  The resulting
+    configuration is set as the global configuration.
 
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.Configurable`
     :param dict params: dictionary of command-line argument key to values
     :param str yaml: global configuration file
     :param str filename: location of global configuration file
     :param dict config: global configuration object
     :param bool validate: check configuration after creating
     :return: the new global configuration
+    :returntype: dict
 
     """
 
@@ -143,14 +168,15 @@ def set_default_config(modules, params={}, yaml=None, filename=None,
                 yakonfig.set_global_config(base_config)
                 checker(base_config[mod.config_name], mod.config_name)
 
-    # All done, set the global configuration
+    # All done, normalize and set the global configuration
+    normalize_config(base_config, modules)
     yakonfig.set_global_config(base_config)
     return base_config
 
 @contextlib.contextmanager
 def defaulted_config(modules, params={}, yaml=None, filename=None,
                      config=None, validate=True):
-    """Context manager version of `set_default_config()`.
+    """Context manager version of :func:`set_default_config()`.
 
     Use this with a Python 'with' statement, like
 
@@ -165,7 +191,8 @@ def defaulted_config(modules, params={}, yaml=None, filename=None,
     On exit the global configuration is restored to its previous state
     (if any).
 
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.Configurable`
     :param dict params: dictionary of command-line argument key to values
     :param str yaml: global configuration file
     :param str filename: location of global configuration file
@@ -182,16 +209,18 @@ def defaulted_config(modules, params={}, yaml=None, filename=None,
 def check_toplevel_config(what, who):
     """Verify that some dependent configuration is present and correct.
 
-    This will generally be called from a `check_config` implementation.
-    `what` is a Configurable-like object.  If the corresponding
-    configuration isn't present in the global configuration, raise a
-    `ConfigurationError` explaining that `who` required it.  Otherwise
-    call that module's `check_config` (if any).
+    This will generally be called from a
+    :meth:`~yakonfig.Configurable.check_config` implementation.
+    `what` is a :class:`~yakonfig.Configurable`-like object.  If the
+    corresponding configuration isn't present in the global
+    configuration, raise a :exc:`yakonfig.ConfigurationError`
+    explaining that `who` required it.  Otherwise call that module's
+    :meth:`~yakonfig.Configurable.check_config` (if any).
 
-    :param Configurable what: top-level module to require
+    :param yakonfig.Configurable what: top-level module to require
     :param str who: name of the requiring module
-    :raise ConfigurationError: if configuration for `what` is missing
-      or incorrect
+    :raise yakonfig.ConfigurationError: if configuration for
+      `what` is missing or incorrect
 
     """
     config_name = what.config_name
@@ -220,7 +249,8 @@ def _walk_config(config, modules, f, prefix='', required=True):
     the default configuration.
 
     :param dict config: configuration to walk and possibly update
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.configurable.Configurable`
     :param f: callback function for each module
     :param str prefix: prefix name of the config
     :param bool required: if true, config names already exist
@@ -260,12 +290,15 @@ def _walk_config(config, modules, f, prefix='', required=True):
 def collect_add_argparse(parser, modules):
     """Add all command-line options.
 
-    `modules` is an iterable of `yakonfig.Configurable` objects, or
-    anything equivalently typed.  This calls `add_arguments` (if
+    `modules` is an iterable of
+    :class:`yakonfig.configurable.Configurable` objects, or anything
+    equivalently typed.  This calls
+    :meth:`~yakonfig.configurable.Configurable.add_arguments` (if
     present) on all of them to set the global command-line arguments.
 
     :param argparse.ArgumentParser parser: argparse parser
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.configurable.Configurable`
 
     """
     def work_in(config, module, name):
@@ -281,11 +314,13 @@ def assemble_default_config(modules):
 
     """Build the default configuration from a set of modules.
 
-    `modules` is an iterable of `yakonfig.Configurable` objects, or
-    anything equivalently typed.  This produces the default
-    configuration from that list of modules.
+    `modules` is an iterable of
+    :class:`yakonfig.configurable.Configurable` objects, or anything
+    equivalently typed.  This produces the default configuration from
+    that list of modules.
 
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.configurable.Configurable`
     :return: configuration dictionary
 
     """
@@ -298,15 +333,16 @@ def fill_in_arguments(config, modules, args):
     """Fill in configuration fields from command-line arguments.
 
     `config` is a dictionary holding the initial configuration,
-    probably the result of `assemble_default_config()`.  It reads
+    probably the result of :func:`assemble_default_config`.  It reads
     through `modules`, and for each, fills in any configuration values
     that are provided in `args`.
 
     `config` is modified in place.  `args` may be either a dictionary
-    or an object (as the result of argparse).
+    or an object (as the result of :mod:`argparse`).
 
     :param dict config: configuration tree to update
-    :param iterable modules: modules or Configurable instances to use
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.configurable.Configurable`
     :param args: command-line objects
     :paramtype args: dict or object
     :return: config
@@ -321,3 +357,23 @@ def fill_in_arguments(config, modules, args):
     if not isinstance(args, collections.Mapping):
         args = vars(args)
     return _walk_config(config, modules, work_in)
+
+def normalize_config(config, modules):
+    """Normalize configuration values in the entire tree.
+
+    `config` is a dictionary holding the almost-final configuration.
+    Each module's
+    :method:`yakonfig.configurable.Configurable.normalize_config`
+    function is called to make changes such as pushing configuration
+    into sub-module configuration blocks and making file paths
+    absolute.
+
+    :param dict config: configuration tree to update
+    :param modules: modules or Configurable instances to use
+    :type modules: iterable of :class:`~yakonfig.configurable.Configurable`
+
+    """
+    def work_in(config, module, name):
+        f = getattr(module, 'normalize_config', None)
+        if f: f(config)
+    return _walk_config(config, modules, work_in)
